/**
 * OpenGL 3.x with GLEW and GLUT
 *
 * author	Po-Ying Li bylee@nchc.org.tw
 * version	1.0
 *
 * Homepage: http://viml.nchc.org.tw
 *
 * Copyright Po-Ying Li
 */
#ifndef _MYNAVIGATION
#define _MYNAVIGATION
#include "math.h"

struct Camera
{
	Vector Eye;
	Vector Center;
	Vector Upper;
	Camera()
	{
		Eye.v[1] = -5.0f;
		Center.v[1] = 0.0f;
		Upper.v[2] = 5.0f;
	}
};

void LookAt(Matrix& mResult,const Vector& vEye,const Vector& vCenter,const Vector& vUpper)
{
	Vector vForward;
	Vector vForwardNorm;
	Vector vSide;
	Vector vUpperNorm;
	Vector vSideNorm; 

	vForward.v[0] = vCenter.v[0] - vEye.v[0];
	vForward.v[1] = vCenter.v[1] - vEye.v[1];
	vForward.v[2] = vCenter.v[2] - vEye.v[2];
	Normalize(vForwardNorm,vForward);
	Cross(vSide,vForwardNorm,vUpper);
	Normalize(vSideNorm,vSide);
	Cross(vUpperNorm,vSideNorm,vForwardNorm);

	Matrix mTmp;
	Matrix mTransltate;

	mTmp.m44[0][0] = vSideNorm.v[0];
	mTmp.m44[1][0] = vSideNorm.v[1];
	mTmp.m44[2][0] = vSideNorm.v[2];
	mTmp.m44[3][0] = 0.0f;

	mTmp.m44[0][1] = vUpperNorm.v[0];
	mTmp.m44[1][1] = vUpperNorm.v[1];
	mTmp.m44[2][1] = vUpperNorm.v[2];
	mTmp.m44[3][1] = 0.0f;

	mTmp.m44[0][2] = -vForwardNorm.v[0];
	mTmp.m44[1][2] = -vForwardNorm.v[1];
	mTmp.m44[2][2] = -vForwardNorm.v[2];
	mTmp.m44[3][2] = 0.0f;

	mTmp.m44[0][3] = 0;
	mTmp.m44[1][3] = 0;
	mTmp.m44[2][3] = 0;
	mTmp.m44[3][3] = 1.0;


	mTransltate.m44[0][0] = 1.0f;
	mTransltate.m44[0][1] = 0.0f;
	mTransltate.m44[0][2] = 0.0f;
	mTransltate.m44[0][3] = 0.0f;

	mTransltate.m44[1][0] = 0.0f;
	mTransltate.m44[1][1] = 1.0f;
	mTransltate.m44[1][2] = 0.0f;
	mTransltate.m44[1][3] = 0.0f;

	mTransltate.m44[2][0] = 0.0f;
	mTransltate.m44[2][1] = 0.0f;
	mTransltate.m44[2][2] = 1.0f;
	mTransltate.m44[2][3] = 0.0f;

	mTransltate.m44[3][0] = -vEye.v[0];
	mTransltate.m44[3][1] = -vEye.v[1];
	mTransltate.m44[3][2] = -vEye.v[2];
	mTransltate.m44[3][3] = 1.0;

	MultMatrix(mResult,mTransltate,mTmp);
}

void Forward(Matrix& mResult, Camera& cCamera)
{
	Vector vForward;
	Vector vForwardNorm;

	Diff(vForward,cCamera.Center,cCamera.Eye);
	Normalize(vForwardNorm,vForward);
	Add(cCamera.Eye,cCamera.Eye,vForwardNorm);
	Add(cCamera.Center,cCamera.Center,vForwardNorm);
	LookAt(mResult,cCamera.Eye,cCamera.Center,cCamera.Upper);
}

void Backward(Matrix& mResult, Camera& cCamera)
{
	Vector vForward;
	Vector vForwardNorm;

	Diff(vForward,cCamera.Center,cCamera.Eye);
	Normalize(vForwardNorm,vForward);
	Diff(cCamera.Eye,cCamera.Eye,vForwardNorm);
	Diff(cCamera.Center,cCamera.Center,vForwardNorm);
	LookAt(mResult,cCamera.Eye,cCamera.Center,cCamera.Upper);
}

void Turn(Matrix& mResult, Camera& cCamera, float fAngle)
{
	Vector vForward;
	Matrix mRotate;
	Vector vDir;

	Diff(vForward,cCamera.Center,cCamera.Eye);
	Rotate(mRotate,fAngle,cCamera.Upper);
	Mult(vDir,mRotate,vForward);
	Add(cCamera.Center,cCamera.Eye,vDir);
	LookAt(mResult,cCamera.Eye,cCamera.Center,cCamera.Upper);

}
void TurnLeft(Matrix& mResult, Camera& cCamera)
{
	Turn(mResult,cCamera,-5);
}
void TurnRight(Matrix& mResult, Camera& cCamera)
{
	Turn(mResult,cCamera,5);
}

#endif